function [nls, nGC, nNLA, nBars, nResidual] = GCD(input, image_type, rho, theta, k)

%path2gs = 'C:\Program Files\gs\gs9.18\bin\gswin64c.exe';
path2gs = '/usr/bin/gs';

if isdeployed
    rho = str2double(rho);
    theta = str2double(theta);
    k = str2double(k);
    
end

%% Read input image
X=imread([input '.' image_type]);
if(size(X,3)>1)
    X = rgb2gray(X);
end
X = double(X);
[m, n] = size(X);
diag = sqrt(m^2+n^2);
%% Create segment structure
Segments = struct('strtip', [], 'endtip', [], 'len', [], 'width', [], 'type', [], 'mdlpnts', [], 'par', []);
nS = 0;
%% LSD
output_lsd=lsd(X);
nls = size(output_lsd,1);
for i=1:nls
    tip1 = [output_lsd(i,1) output_lsd(i,2)];
    tip2 = [output_lsd(i,3) output_lsd(i,4)];
    [Segments, nS] = fillSegmentsStruct(Segments, nS, tip1, tip2, norm(tip1-tip2), 1, [], [], diag);
end
write_eps(Segments, m, n, [input '_LSD.eps'], 1);
eps2xxx([input '_LSD.eps'],{'png'},path2gs,1);
delete([input '_LSD.eps']);

% Make Adjacency Matrix
adjM = makeAdjMatrix(Segments,rho,theta);

% none local alignments

lstruct = findNLA(adjM,k,m,n);
lstruct = sortChains(lstruct);
lc = LineChain(nS, lstruct);
lstruct = findCurves(lstruct,lc);
[Segments, nS] = addSeg(Segments, nS, lstruct, diag, 2);
write_eps(Segments, m, n, [input '_NLA.eps'], 2);
eps2xxx([input '_NLA.eps'],{'png'},path2gs);
nNLA = sum([Segments.type]==2);
delete([input '_NLA.eps']);

% Find Good continuations
chains = findChains(adjM,k,m,n);
chains = sortChains(chains); %length, rho, theta, nfa
lc = LineChain(sum([Segments.type]==1), chains);
chains = findCurves(chains,lc, adjM);
[Segments, nS] = addSeg(Segments, nS, chains, diag, 3);
write_eps(Segments, m, n, [input '_GC.eps'], 3);
eps2xxx([input '_GC.eps'],{'png'},path2gs);
nGC = sum([Segments.type]==3);
delete([input '_GC.eps']);


%% Bars
adjM_Bar = makeAdjMatrix_Bar(Segments,rho);
BarsStruct = CalBars(adjM_Bar,m,n);
Bars = findBars(BarsStruct, sum([Segments.type]<3));
[Segments, nS] = addSeg(Segments, nS, Bars, diag, 4);
write_eps(Segments, m, n, [input '_Bars.eps'], 4);
eps2xxx([input '_Bars.eps'],{'png'},path2gs);
nBars = sum([Segments.type]==4);
delete([input '_Bars.eps']);

%% Residual
write_residual_eps(Segments, m, n, [input '_Residual.eps'],[Segments.par]);
eps2xxx([input '_Residual.eps'],{'png'},path2gs);
delete([input '_Residual.eps']);
nResidual = sum([Segments.type]==1) - length(unique([Segments.par]));

%% Write web interface variables
fid = fopen('./algo_info.txt', 'w');
fwrite(fid, sprintf('nls=%d\n', nls));
fwrite(fid, sprintf('nGC=%d\n', nGC));
fwrite(fid, sprintf('nBars=%d\n', nBars));
fwrite(fid, sprintf('nNLA=%d\n', nNLA));
fwrite(fid, sprintf('nResidual=%d\n', nResidual));
fclose(fid);

